@@ -5,8 +5,8 @@ use bench::criterion::{criterion_group, criterion_main, measurement::WallTime, C
 use pypi_types::Requirement;
 use uv_cache::Cache;
 use uv_client::RegistryClientBuilder;
-use uv_interpreter::PythonEnvironment;
 use uv_resolver::Manifest;
+use uv_toolchain::PythonEnvironment;
 
 fn resolve_warm_jupyter(c: &mut Criterion<WallTime>) {
     let runtime = &tokio::runtime::Builder::new_current_thread()
@@ -15,7 +15,7 @@ fn resolve_warm_jupyter(c: &mut Criterion<WallTime>) {
         .unwrap();
 
     let cache = &Cache::from_path("../../.cache").init().unwrap();
-    let venv = PythonEnvironment::from_virtualenv(cache).unwrap();
+    let venv = PythonEnvironment::from_root("../../.venv", cache).unwrap();
     let client = &RegistryClientBuilder::new(cache.clone()).build();
     let manifest = &Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("jupyter").unwrap(),
@@ -44,7 +44,7 @@ fn resolve_warm_airflow(c: &mut Criterion<WallTime>) {
         .unwrap();
 
     let cache = &Cache::from_path("../../.cache").init().unwrap();
-    let venv = PythonEnvironment::from_virtualenv(cache).unwrap();
+    let venv = PythonEnvironment::from_root("../../.venv", cache).unwrap();
     let client = &RegistryClientBuilder::new(cache.clone()).build();
     let manifest = &Manifest::simple(vec![
         Requirement::from(pep508_rs::Requirement::from_str("apache-airflow[all]").unwrap()),
@@ -86,10 +86,10 @@ mod resolver {
     use uv_dispatch::BuildDispatch;
     use uv_distribution::DistributionDatabase;
     use uv_git::GitResolver;
-    use uv_interpreter::PythonEnvironment;
     use uv_resolver::{
         FlatIndex, InMemoryIndex, Manifest, Options, PythonRequirement, ResolutionGraph, Resolver,
     };
+    use uv_toolchain::PythonEnvironment;
     use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 
     static MARKERS: Lazy<MarkerEnvironment> = Lazy::new(|| {
@@ -135,13 +135,13 @@ mod resolver {
         let index = InMemoryIndex::default();
         let index_locations = IndexLocations::default();
         let installed_packages = EmptyInstalledPackages;
-        let interpreter = venv.interpreter().clone();
-        let python_requirement = PythonRequirement::from_marker_environment(&interpreter, &MARKERS);
+        let interpreter = venv.interpreter();
+        let python_requirement = PythonRequirement::from_interpreter(interpreter);
 
         let build_context = BuildDispatch::new(
             client,
             &cache,
-            &interpreter,
+            interpreter,
             &index_locations,
             &flat_index,
             &index,
@@ -162,7 +162,7 @@ mod resolver {
             Options::default(),
             &python_requirement,
             Some(&MARKERS),
-            &TAGS,
+            Some(&TAGS),
             &flat_index,
             &index,
             &hashes,
