@@ -44,7 +44,6 @@ use tracing::instrument;
 use unscanny::{Pattern, Scanner};
 use url::Url;
 
-use crate::requirement::EditableError;
 use distribution_types::{UnresolvedRequirement, UnresolvedRequirementSpecification};
 use pep508_rs::{
     expand_env_vars, split_scheme, strip_host, Pep508Error, RequirementOrigin, Scheme, VerbatimUrl,
@@ -57,11 +56,12 @@ use uv_configuration::{NoBinary, NoBuild, PackageNameSpecifier};
 use uv_fs::{normalize_url_path, Simplified};
 use uv_warnings::warn_user;
 
+use crate::requirement::EditableError;
 pub use crate::requirement::RequirementsTxtRequirement;
 
 mod requirement;
 
-/// We emit one of those for each requirements.txt entry
+/// We emit one of those for each `requirements.txt` entry.
 enum RequirementsTxtStatement {
     /// `-r` inclusion filename
     Requirements {
@@ -262,6 +262,7 @@ impl RequirementsTxt {
                     read_url_to_string(&requirements_txt, client).await
                 }
             } else {
+                // Ex) `file:///home/ferris/project/requirements.txt`
                 uv_fs::read_to_string_transcode(&requirements_txt)
                     .await
                     .map_err(RequirementsTxtParserError::IO)
@@ -321,6 +322,22 @@ impl RequirementsTxt {
                     let sub_file =
                         if filename.starts_with("http://") || filename.starts_with("https://") {
                             PathBuf::from(filename.as_ref())
+                        } else if filename.starts_with("file://") {
+                            requirements_txt.join(
+                                Url::parse(filename.as_ref())
+                                    .map_err(|err| RequirementsTxtParserError::Url {
+                                        source: err,
+                                        url: filename.to_string(),
+                                        start,
+                                        end,
+                                    })?
+                                    .to_file_path()
+                                    .map_err(|()| RequirementsTxtParserError::FileUrl {
+                                        url: filename.to_string(),
+                                        start,
+                                        end,
+                                    })?,
+                            )
                         } else {
                             requirements_dir.join(filename.as_ref())
                         };
@@ -360,6 +377,22 @@ impl RequirementsTxt {
                     let sub_file =
                         if filename.starts_with("http://") || filename.starts_with("https://") {
                             PathBuf::from(filename.as_ref())
+                        } else if filename.starts_with("file://") {
+                            requirements_txt.join(
+                                Url::parse(filename.as_ref())
+                                    .map_err(|err| RequirementsTxtParserError::Url {
+                                        source: err,
+                                        url: filename.to_string(),
+                                        start,
+                                        end,
+                                    })?
+                                    .to_file_path()
+                                    .map_err(|()| RequirementsTxtParserError::FileUrl {
+                                        url: filename.to_string(),
+                                        start,
+                                        end,
+                                    })?,
+                            )
                         } else {
                             requirements_dir.join(filename.as_ref())
                         };
@@ -500,7 +533,8 @@ fn parse_entry(
             Some(requirements_txt)
         };
 
-        let (requirement, hashes) = parse_requirement_and_hashes(s, content, source, working_dir)?;
+        let (requirement, hashes) =
+            parse_requirement_and_hashes(s, content, source, working_dir, true)?;
         let requirement =
             requirement
                 .into_editable()
@@ -579,7 +613,8 @@ fn parse_entry(
             Some(requirements_txt)
         };
 
-        let (requirement, hashes) = parse_requirement_and_hashes(s, content, source, working_dir)?;
+        let (requirement, hashes) =
+            parse_requirement_and_hashes(s, content, source, working_dir, false)?;
         RequirementsTxtStatement::RequirementEntry(RequirementEntry {
             requirement,
             hashes,
@@ -643,6 +678,7 @@ fn parse_requirement_and_hashes(
     content: &str,
     source: Option<&Path>,
     working_dir: &Path,
+    editable: bool,
 ) -> Result<(RequirementsTxtRequirement, Vec<String>), RequirementsTxtParserError> {
     // PEP 508 requirement
     let start = s.cursor();
@@ -699,7 +735,7 @@ fn parse_requirement_and_hashes(
         }
     }
 
-    let requirement = RequirementsTxtRequirement::parse(requirement, working_dir)
+    let requirement = RequirementsTxtRequirement::parse(requirement, working_dir, editable)
         .map(|requirement| {
             if let Some(source) = source {
                 requirement.with_origin(RequirementOrigin::File(source.to_path_buf()))
@@ -812,6 +848,11 @@ pub enum RequirementsTxtParserError {
         start: usize,
         end: usize,
     },
+    FileUrl {
+        url: String,
+        start: usize,
+        end: usize,
+    },
     VerbatimUrl {
         source: pep508_rs::VerbatimUrlError,
         url: String,
@@ -879,6 +920,9 @@ impl Display for RequirementsTxtParserError {
             Self::Url { url, start, .. } => {
                 write!(f, "Invalid URL at position {start}: `{url}`")
             }
+            Self::FileUrl { url, start, .. } => {
+                write!(f, "Invalid file URL at position {start}: `{url}`")
+            }
             Self::VerbatimUrl { source, url } => {
                 write!(f, "Invalid URL: `{url}`: {source}")
             }
@@ -942,6 +986,7 @@ impl std::error::Error for RequirementsTxtParserError {
         match &self {
             Self::IO(err) => err.source(),
             Self::Url { source, .. } => Some(source),
+            Self::FileUrl { .. } => None,
             Self::VerbatimUrl { source, .. } => Some(source),
             Self::UrlConversion(_) => None,
             Self::UnsupportedUrl(_) => None,
@@ -973,6 +1018,13 @@ impl Display for RequirementsTxtFileError {
                     self.file.user_display(),
                 )
             }
+            RequirementsTxtParserError::FileUrl { url, start, .. } => {
+                write!(
+                    f,
+                    "Invalid file URL in `{}` at position {start}: `{url}`",
+                    self.file.user_display(),
+                )
+            }
             RequirementsTxtParserError::VerbatimUrl { url, .. } => {
                 write!(f, "Invalid URL in `{}`: `{url}`", self.file.user_display())
             }
