@@ -15,8 +15,8 @@ use uv_configuration::{
 };
 use uv_dispatch::BuildDispatch;
 use uv_git::GitResolver;
-use uv_interpreter::PythonEnvironment;
 use uv_resolver::{FlatIndex, InMemoryIndex};
+use uv_toolchain::Toolchain;
 use uv_types::{BuildContext, BuildIsolation, InFlight};
 
 #[derive(Parser)]
@@ -65,12 +65,12 @@ pub(crate) async fn build(args: BuildArgs) -> Result<PathBuf> {
     let index = InMemoryIndex::default();
     let index_urls = IndexLocations::default();
     let setup_py = SetupPyStrategy::default();
-    let venv = PythonEnvironment::from_virtualenv(&cache)?;
+    let toolchain = Toolchain::find_virtualenv(&cache)?;
 
     let build_dispatch = BuildDispatch::new(
         &client,
         &cache,
-        venv.interpreter(),
+        toolchain.interpreter(),
         &index_urls,
         &flat_index,
         &index,
