@@ -11,9 +11,10 @@ use distribution_filename::WheelFilename;
 use distribution_types::{
     CachedDirectUrlDist, CachedDist, DirectUrlBuiltDist, DirectUrlSourceDist, DirectorySourceDist,
     Error, GitSourceDist, Hashed, IndexLocations, InstalledDist, Name, PathBuiltDist,
-    PathSourceDist, RemoteSource, Requirement, RequirementSource, Verbatim,
+    PathSourceDist, RemoteSource, Verbatim,
 };
 use platform_tags::Tags;
+use pypi_types::{Requirement, RequirementSource};
 use uv_cache::{ArchiveTimestamp, Cache, CacheBucket, WheelCache};
 use uv_configuration::{NoBinary, Reinstall};
 use uv_distribution::{
@@ -21,7 +22,7 @@ use uv_distribution::{
 };
 use uv_fs::Simplified;
 use uv_git::GitUrl;
-use uv_interpreter::PythonEnvironment;
+use uv_toolchain::PythonEnvironment;
 use uv_types::HashStrategy;
 
 use crate::satisfies::RequirementSatisfaction;
@@ -112,16 +113,18 @@ impl<'a> Planner<'a> {
                 NoBinary::Packages(packages) => packages.contains(&requirement.name),
             };
 
+            let installed_dists = site_packages.remove_packages(&requirement.name);
+
             if reinstall {
-                let installed_dists = site_packages.remove_packages(&requirement.name);
                 reinstalls.extend(installed_dists);
             } else {
-                let installed_dists = site_packages.remove_packages(&requirement.name);
                 match installed_dists.as_slice() {
                     [] => {}
                     [distribution] => {
                         match RequirementSatisfaction::check(distribution, &requirement.source)? {
-                            RequirementSatisfaction::Mismatch => {}
+                            RequirementSatisfaction::Mismatch => {
+                                debug!("Requirement installed, but mismatched: {distribution:?}");
+                            }
                             RequirementSatisfaction::Satisfied => {
                                 debug!("Requirement already installed: {distribution}");
                                 continue;
