@@ -13,12 +13,13 @@ use pubgrub::type_aliases::Map;
 use rustc_hash::FxHashMap;
 
 use distribution_types::IndexLocations;
-use pep440_rs::{Version, VersionSpecifiers};
+use pep440_rs::Version;
 use uv_normalize::PackageName;
 
 use crate::candidate_selector::CandidateSelector;
-use crate::python_requirement::{PythonRequirement, RequiresPython};
+use crate::python_requirement::{PythonRequirement, PythonTarget};
 use crate::resolver::{IncompletePackage, UnavailablePackage, UnavailableReason};
+use crate::RequiresPython;
 
 use super::{PubGrubPackage, PubGrubPackageInner, PubGrubPython};
 
@@ -407,67 +408,64 @@ impl PubGrubReportFormatter<'_> {
         unavailable_packages: &FxHashMap<PackageName, UnavailablePackage>,
         incomplete_packages: &FxHashMap<PackageName, BTreeMap<Version, IncompletePackage>>,
     ) -> IndexSet<PubGrubHint> {
-        /// Returns `true` if pre-releases were allowed for a package.
-        fn allowed_prerelease(package: &PubGrubPackage, selector: &CandidateSelector) -> bool {
-            let PubGrubPackageInner::Package { name, .. } = &**package else {
-                return false;
-            };
-            selector.prerelease_strategy().allows(name)
-        }
-
         let mut hints = IndexSet::default();
         match derivation_tree {
             DerivationTree::External(external) => match external {
                 External::Custom(package, set, _) | External::NoVersions(package, set) => {
-                    // Check for no versions due to pre-release options
-                    if let Some(selector) = selector {
-                        let any_prerelease = set.iter().any(|(start, end)| {
-                            let is_pre1 = match start {
-                                Bound::Included(version) => version.any_prerelease(),
-                                Bound::Excluded(version) => version.any_prerelease(),
-                                Bound::Unbounded => false,
-                            };
-                            let is_pre2 = match end {
-                                Bound::Included(version) => version.any_prerelease(),
-                                Bound::Excluded(version) => version.any_prerelease(),
-                                Bound::Unbounded => false,
-                            };
-                            is_pre1 || is_pre2
-                        });
+                    if let PubGrubPackageInner::Package {
+                        name, url: None, ..
+                    } = &**package
+                    {
+                        // Check for no versions due to pre-release options.
+                        if let Some(selector) = selector {
+                            let any_prerelease = set.iter().any(|(start, end)| {
+                                let is_pre1 = match start {
+                                    Bound::Included(version) => version.any_prerelease(),
+                                    Bound::Excluded(version) => version.any_prerelease(),
+                                    Bound::Unbounded => false,
+                                };
+                                let is_pre2 = match end {
+                                    Bound::Included(version) => version.any_prerelease(),
+                                    Bound::Excluded(version) => version.any_prerelease(),
+                                    Bound::Unbounded => false,
+                                };
+                                is_pre1 || is_pre2
+                            });
 
-                        if any_prerelease {
-                            // A pre-release marker appeared in the version requirements.
-                            if !allowed_prerelease(package, selector) {
-                                hints.insert(PubGrubHint::PreReleaseRequested {
-                                    package: package.clone(),
-                                    range: self.simplify_set(set, package).into_owned(),
-                                });
-                            }
-                        } else if let Some(version) =
-                            self.available_versions.get(package).and_then(|versions| {
-                                versions
-                                    .iter()
-                                    .rev()
-                                    .filter(|version| version.any_prerelease())
-                                    .find(|version| set.contains(version))
-                            })
-                        {
-                            // There are pre-release versions available for the package.
-                            if !allowed_prerelease(package, selector) {
-                                hints.insert(PubGrubHint::PreReleaseAvailable {
-                                    package: package.clone(),
-                                    version: version.clone(),
-                                });
+                            if any_prerelease {
+                                // A pre-release marker appeared in the version requirements.
+                                if !selector.prerelease_strategy().allows(name) {
+                                    hints.insert(PubGrubHint::PreReleaseRequested {
+                                        package: package.clone(),
+                                        range: self.simplify_set(set, package).into_owned(),
+                                    });
+                                }
+                            } else if let Some(version) =
+                                self.available_versions.get(package).and_then(|versions| {
+                                    versions
+                                        .iter()
+                                        .rev()
+                                        .filter(|version| version.any_prerelease())
+                                        .find(|version| set.contains(version))
+                                })
+                            {
+                                // There are pre-release versions available for the package.
+                                if !selector.prerelease_strategy().allows(name) {
+                                    hints.insert(PubGrubHint::PreReleaseAvailable {
+                                        package: package.clone(),
+                                        version: version.clone(),
+                                    });
+                                }
                             }
                         }
                     }
 
-                    // Check for no versions due to no `--find-links` flat index
-                    if let Some(index_locations) = index_locations {
-                        let no_find_links =
-                            index_locations.flat_index().peekable().peek().is_none();
+                    if let PubGrubPackageInner::Package { name, .. } = &**package {
+                        // Check for no versions due to no `--find-links` flat index
+                        if let Some(index_locations) = index_locations {
+                            let no_find_links =
+                                index_locations.flat_index().peekable().peek().is_none();
 
-                        if let PubGrubPackageInner::Package { name, .. } = &**package {
                             // Add hints due to the package being entirely unavailable.
                             match unavailable_packages.get(name) {
                                 Some(UnavailablePackage::NoIndex) => {
@@ -542,9 +540,11 @@ impl PubGrubReportFormatter<'_> {
                         PubGrubPackageInner::Python(PubGrubPython::Target)
                     ) {
                         if let Some(python) = self.python_requirement {
-                            if let Some(RequiresPython::Specifiers(specifiers)) = python.target() {
+                            if let Some(PythonTarget::RequiresPython(requires_python)) =
+                                python.target()
+                            {
                                 hints.insert(PubGrubHint::RequiresPython {
-                                    requires_python: specifiers.clone(),
+                                    requires_python: requires_python.clone(),
                                     package: package.clone(),
                                     package_set: self
                                         .simplify_set(package_set, package)
@@ -640,7 +640,7 @@ pub(crate) enum PubGrubHint {
     },
     /// The `Requires-Python` requirement was not satisfied.
     RequiresPython {
-        requires_python: VersionSpecifiers,
+        requires_python: RequiresPython,
         #[derivative(PartialEq = "ignore", Hash = "ignore")]
         package: PubGrubPackage,
         #[derivative(PartialEq = "ignore", Hash = "ignore")]
