@@ -74,7 +74,7 @@ pub struct DefaultResolverProvider<'a, Context: BuildContext> {
     fetcher: DistributionDatabase<'a, Context>,
     /// These are the entries from `--find-links` that act as overrides for index responses.
     flat_index: FlatIndex,
-    tags: Tags,
+    tags: Option<Tags>,
     python_requirement: PythonRequirement,
     allowed_yanks: AllowedYanks,
     hasher: HashStrategy,
@@ -89,7 +89,7 @@ impl<'a, Context: BuildContext> DefaultResolverProvider<'a, Context> {
     pub fn new(
         fetcher: DistributionDatabase<'a, Context>,
         flat_index: &'a FlatIndex,
-        tags: &'a Tags,
+        tags: Option<&'a Tags>,
         python_requirement: PythonRequirement,
         allowed_yanks: AllowedYanks,
         hasher: &'a HashStrategy,
@@ -100,7 +100,7 @@ impl<'a, Context: BuildContext> DefaultResolverProvider<'a, Context> {
         Self {
             fetcher,
             flat_index: flat_index.clone(),
-            tags: tags.clone(),
+            tags: tags.cloned(),
             python_requirement,
             allowed_yanks,
             hasher: hasher.clone(),
@@ -132,7 +132,7 @@ impl<'a, Context: BuildContext> ResolverProvider for DefaultResolverProvider<'a,
                             metadata,
                             package_name,
                             &index,
-                            &self.tags,
+                            self.tags.as_ref(),
                             &self.python_requirement,
                             &self.allowed_yanks,
                             &self.hasher,
