@@ -1,9 +1,12 @@
-use std::fmt::{self, Display};
+use std::fmt::Display;
 use std::io;
+use std::num::ParseIntError;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
-use crate::PythonVersion;
+use crate::implementation::{Error as ImplementationError, ImplementationName};
+use crate::platform::{Arch, Error as PlatformError, Libc, Os};
+use crate::{PythonVersion, ToolchainRequest, VersionRequest};
 use thiserror::Error;
 use uv_client::BetterReqwestError;
 
@@ -16,36 +19,43 @@ use uv_fs::Simplified;
 
 #[derive(Error, Debug)]
 pub enum Error {
-    #[error("operating system not supported: {0}")]
-    OsNotSupported(String),
-    #[error("architecture not supported: {0}")]
-    ArchNotSupported(String),
-    #[error("libc type could not be detected")]
-    LibcNotDetected(),
-    #[error("invalid python version: {0}")]
-    InvalidPythonVersion(String),
-    #[error("download failed")]
-    NetworkError(#[from] BetterReqwestError),
-    #[error("download failed")]
-    NetworkMiddlewareError(#[source] anyhow::Error),
     #[error(transparent)]
-    ExtractError(#[from] uv_extract::Error),
-    #[error("invalid download url")]
+    IO(#[from] io::Error),
+    #[error(transparent)]
+    PlatformError(#[from] PlatformError),
+    #[error(transparent)]
+    ImplementationError(#[from] ImplementationError),
+    #[error("Invalid python version: {0}")]
+    InvalidPythonVersion(ParseIntError),
+    #[error("Download failed")]
+    NetworkError(#[from] BetterReqwestError),
+    #[error("Download failed")]
+    NetworkMiddlewareError(#[source] anyhow::Error),
+    #[error("Failed to extract archive: {0}")]
+    ExtractError(String, #[source] uv_extract::Error),
+    #[error("Invalid download url")]
     InvalidUrl(#[from] url::ParseError),
-    #[error("failed to create download directory")]
+    #[error("Failed to create download directory")]
     DownloadDirError(#[source] io::Error),
-    #[error("failed to copy to: {0}", to.user_display())]
+    #[error("Failed to copy to: {0}", to.user_display())]
     CopyError {
         to: PathBuf,
         #[source]
         err: io::Error,
     },
-    #[error("failed to read toolchain directory: {0}", dir.user_display())]
+    #[error("Failed to read toolchain directory: {0}", dir.user_display())]
     ReadError {
         dir: PathBuf,
         #[source]
         err: io::Error,
     },
+    #[error("Failed to parse toolchain directory name: {0}")]
+    NameError(String),
+    #[error("Cannot download toolchain for request: {0}")]
+    InvalidRequestKind(ToolchainRequest),
+    // TODO(zanieb): Implement display for `PythonDownloadRequest`
+    #[error("No download found for request: {0:?}")]
+    NoDownloadFound(PythonDownloadRequest),
 }
 
 #[derive(Debug, PartialEq)]
@@ -62,9 +72,9 @@ pub struct PythonDownload {
     sha256: Option<&'static str>,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone, Default)]
 pub struct PythonDownloadRequest {
-    version: Option<PythonVersion>,
+    version: Option<VersionRequest>,
     implementation: Option<ImplementationName>,
     arch: Option<Arch>,
     os: Option<Os>,
@@ -73,7 +83,7 @@ pub struct PythonDownloadRequest {
 
 impl PythonDownloadRequest {
     pub fn new(
-        version: Option<PythonVersion>,
+        version: Option<VersionRequest>,
         implementation: Option<ImplementationName>,
         arch: Option<Arch>,
         os: Option<Os>,
@@ -94,6 +104,12 @@ impl PythonDownloadRequest {
         self
     }
 
+    #[must_use]
+    pub fn with_version(mut self, version: VersionRequest) -> Self {
+        self.version = Some(version);
+        self
+    }
+
     #[must_use]
     pub fn with_arch(mut self, arch: Arch) -> Self {
         self.arch = Some(arch);
@@ -112,9 +128,34 @@ impl PythonDownloadRequest {
         self
     }
 
+    /// Construct a new [`PythonDownloadRequest`] from a [`ToolchainRequest`].
+    pub fn from_request(request: ToolchainRequest) -> Result<Self, Error> {
+        let result = Self::default();
+        let result = match request {
+            ToolchainRequest::Version(version) => result.with_version(version),
+            ToolchainRequest::Implementation(implementation) => {
+                result.with_implementation(implementation)
+            }
+            ToolchainRequest::ImplementationVersion(implementation, version) => result
+                .with_implementation(implementation)
+                .with_version(version),
+            ToolchainRequest::Any => result,
+            // We can't download a toolchain for these request kinds
+            ToolchainRequest::Directory(_)
+            | ToolchainRequest::ExecutableName(_)
+            | ToolchainRequest::File(_) => {
+                return Err(Error::InvalidRequestKind(request));
+            }
+        };
+        Ok(result)
+    }
+
+    /// Fill empty entries with default values.
+    ///
+    /// Platform information is pulled from the environment.
     pub fn fill(mut self) -> Result<Self, Error> {
         if self.implementation.is_none() {
-            self.implementation = Some(ImplementationName::Cpython);
+            self.implementation = Some(ImplementationName::CPython);
         }
         if self.arch.is_none() {
             self.arch = Some(Arch::from_env()?);
@@ -123,41 +164,87 @@ impl PythonDownloadRequest {
             self.os = Some(Os::from_env()?);
         }
         if self.libc.is_none() {
-            self.libc = Some(Libc::from_env()?);
+            self.libc = Some(Libc::from_env());
         }
         Ok(self)
     }
+
+    /// Construct a new [`PythonDownloadRequest`] with platform information from the environment.
+    pub fn from_env() -> Result<Self, Error> {
+        Ok(Self::new(
+            None,
+            None,
+            Some(Arch::from_env()?),
+            Some(Os::from_env()?),
+            Some(Libc::from_env()),
+        ))
+    }
+
+    /// Iterate over all [`PythonDownload`]'s that match this request.
+    pub fn iter_downloads(&self) -> impl Iterator<Item = &'static PythonDownload> + '_ {
+        PythonDownload::iter_all().filter(move |download| {
+            if let Some(arch) = &self.arch {
+                if download.arch != *arch {
+                    return false;
+                }
+            }
+            if let Some(os) = &self.os {
+                if download.os != *os {
+                    return false;
+                }
+            }
+            if let Some(implementation) = &self.implementation {
+                if download.implementation != *implementation {
+                    return false;
+                }
+            }
+            if let Some(version) = &self.version {
+                if !version.matches_major_minor_patch(
+                    download.major,
+                    download.minor,
+                    download.patch,
+                ) {
+                    return false;
+                }
+            }
+            true
+        })
+    }
+}
+
+impl Display for PythonDownloadRequest {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut parts = Vec::new();
+        if let Some(version) = self.version {
+            parts.push(version.to_string());
+        }
+        if let Some(implementation) = self.implementation {
+            parts.push(implementation.to_string());
+        }
+        if let Some(os) = &self.os {
+            parts.push(os.to_string());
+        }
+        if let Some(arch) = self.arch {
+            parts.push(arch.to_string());
+        }
+        if let Some(libc) = self.libc {
+            parts.push(libc.to_string());
+        }
+        write!(f, "{}", parts.join("-"))
+    }
 }
 
 impl FromStr for PythonDownloadRequest {
     type Err = Error;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        // TOOD(zanieb): Implement parsing of additional request parts
-        let version = PythonVersion::from_str(s).map_err(Error::InvalidPythonVersion)?;
+        // TODO(zanieb): Implement parsing of additional request parts
+        let version = VersionRequest::from_str(s).map_err(Error::InvalidPythonVersion)?;
         Ok(Self::new(Some(version), None, None, None, None))
     }
 }
 
-#[derive(Debug, PartialEq)]
-pub enum Libc {
-    Gnu,
-    Musl,
-    None,
-}
-
-#[derive(Debug, PartialEq)]
-pub enum ImplementationName {
-    Cpython,
-}
-#[derive(Debug, PartialEq)]
-pub struct Platform {
-    os: Os,
-    arch: Arch,
-    libc: Libc,
-}
-
-include!("python_versions.inc");
+include!("downloads.inc");
 
 pub enum DownloadResult {
     AlreadyAvailable(PathBuf),
@@ -170,45 +257,27 @@ impl PythonDownload {
         PYTHON_DOWNLOADS.iter().find(|&value| value.key == key)
     }
 
-    pub fn from_request(request: &PythonDownloadRequest) -> Option<&'static PythonDownload> {
-        for download in PYTHON_DOWNLOADS {
-            if let Some(arch) = &request.arch {
-                if download.arch != *arch {
-                    continue;
-                }
-            }
-            if let Some(os) = &request.os {
-                if download.os != *os {
-                    continue;
-                }
-            }
-            if let Some(implementation) = &request.implementation {
-                if download.implementation != *implementation {
-                    continue;
-                }
-            }
-            if let Some(version) = &request.version {
-                if download.major != version.major() {
-                    continue;
-                }
-                if download.minor != version.minor() {
-                    continue;
-                }
-                if let Some(patch) = version.patch() {
-                    if download.patch != patch {
-                        continue;
-                    }
-                }
-            }
-            return Some(download);
-        }
-        None
+    /// Return the first [`PythonDownload`] matching a request, if any.
+    pub fn from_request(request: &PythonDownloadRequest) -> Result<&'static PythonDownload, Error> {
+        request
+            .iter_downloads()
+            .next()
+            .ok_or(Error::NoDownloadFound(request.clone()))
+    }
+
+    /// Iterate over all [`PythonDownload`]'s.
+    pub fn iter_all() -> impl Iterator<Item = &'static PythonDownload> {
+        PYTHON_DOWNLOADS.iter()
     }
 
     pub fn url(&self) -> &str {
         self.url
     }
 
+    pub fn key(&self) -> &str {
+        self.key
+    }
+
     pub fn sha256(&self) -> Option<&str> {
         self.sha256
     }
@@ -246,13 +315,15 @@ impl PythonDownload {
             .into_async_read();
 
         debug!("Extracting {filename}");
-        uv_extract::stream::archive(reader.compat(), filename, temp_dir.path()).await?;
+        uv_extract::stream::archive(reader.compat(), filename, temp_dir.path())
+            .await
+            .map_err(|err| Error::ExtractError(filename.to_string(), err))?;
 
         // Extract the top-level directory.
         let extracted = match uv_extract::strip_component(temp_dir.path()) {
             Ok(top_level) => top_level,
             Err(uv_extract::Error::NonSingularArchive(_)) => temp_dir.into_path(),
-            Err(err) => return Err(err.into()),
+            Err(err) => return Err(Error::ExtractError(filename.to_string(), err)),
         };
 
         // Persist it to the target
@@ -273,147 +344,6 @@ impl PythonDownload {
     }
 }
 
-impl Platform {
-    pub fn new(os: Os, arch: Arch, libc: Libc) -> Self {
-        Self { os, arch, libc }
-    }
-    pub fn from_env() -> Result<Self, Error> {
-        Ok(Self::new(
-            Os::from_env()?,
-            Arch::from_env()?,
-            Libc::from_env()?,
-        ))
-    }
-}
-
-/// All supported operating systems.
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub enum Os {
-    Windows,
-    Linux,
-    Macos,
-    FreeBsd,
-    NetBsd,
-    OpenBsd,
-    Dragonfly,
-    Illumos,
-    Haiku,
-}
-
-impl fmt::Display for Os {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            Self::Windows => write!(f, "Windows"),
-            Self::Macos => write!(f, "MacOS"),
-            Self::FreeBsd => write!(f, "FreeBSD"),
-            Self::NetBsd => write!(f, "NetBSD"),
-            Self::Linux => write!(f, "Linux"),
-            Self::OpenBsd => write!(f, "OpenBSD"),
-            Self::Dragonfly => write!(f, "DragonFly"),
-            Self::Illumos => write!(f, "Illumos"),
-            Self::Haiku => write!(f, "Haiku"),
-        }
-    }
-}
-
-impl Os {
-    pub(crate) fn from_env() -> Result<Self, Error> {
-        Self::from_str(std::env::consts::OS)
-    }
-}
-
-impl FromStr for Os {
-    type Err = Error;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_lowercase().as_str() {
-            "windows" => Ok(Self::Windows),
-            "linux" => Ok(Self::Linux),
-            "macos" => Ok(Self::Macos),
-            "freebsd" => Ok(Self::FreeBsd),
-            "netbsd" => Ok(Self::NetBsd),
-            "openbsd" => Ok(Self::OpenBsd),
-            "dragonfly" => Ok(Self::Dragonfly),
-            "illumos" => Ok(Self::Illumos),
-            "haiku" => Ok(Self::Haiku),
-            _ => Err(Error::OsNotSupported(s.to_string())),
-        }
-    }
-}
-
-/// All supported CPU architectures
-#[derive(Debug, Clone, Copy, Eq, PartialEq)]
-pub enum Arch {
-    Aarch64,
-    Armv6L,
-    Armv7L,
-    Powerpc64Le,
-    Powerpc64,
-    X86,
-    X86_64,
-    S390X,
-}
-
-impl fmt::Display for Arch {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            Self::Aarch64 => write!(f, "aarch64"),
-            Self::Armv6L => write!(f, "armv6l"),
-            Self::Armv7L => write!(f, "armv7l"),
-            Self::Powerpc64Le => write!(f, "ppc64le"),
-            Self::Powerpc64 => write!(f, "ppc64"),
-            Self::X86 => write!(f, "i686"),
-            Self::X86_64 => write!(f, "x86_64"),
-            Self::S390X => write!(f, "s390x"),
-        }
-    }
-}
-
-impl FromStr for Arch {
-    type Err = Error;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_lowercase().as_str() {
-            "aarch64" | "arm64" => Ok(Self::Aarch64),
-            "armv6l" => Ok(Self::Armv6L),
-            "armv7l" => Ok(Self::Armv7L),
-            "powerpc64le" | "ppc64le" => Ok(Self::Powerpc64Le),
-            "powerpc64" | "ppc64" => Ok(Self::Powerpc64),
-            "x86" | "i686" | "i386" => Ok(Self::X86),
-            "x86_64" | "amd64" => Ok(Self::X86_64),
-            "s390x" => Ok(Self::S390X),
-            _ => Err(Error::ArchNotSupported(s.to_string())),
-        }
-    }
-}
-
-impl Arch {
-    pub(crate) fn from_env() -> Result<Self, Error> {
-        Self::from_str(std::env::consts::ARCH)
-    }
-}
-
-impl Libc {
-    pub(crate) fn from_env() -> Result<Self, Error> {
-        // TODO(zanieb): Perform this lookup
-        match std::env::consts::OS {
-            "linux" => Ok(Libc::Gnu),
-            "windows" | "macos" => Ok(Libc::None),
-            _ => Err(Error::LibcNotDetected()),
-        }
-    }
-}
-
-impl fmt::Display for Libc {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            Libc::Gnu => f.write_str("gnu"),
-            Libc::None => f.write_str("none"),
-            Libc::Musl => f.write_str("musl"),
-        }
-    }
-}
-
 impl From<reqwest::Error> for Error {
     fn from(error: reqwest::Error) -> Self {
         Self::NetworkError(BetterReqwestError::from(error))
