@@ -3,13 +3,13 @@ use std::path::{Path, PathBuf};
 
 use anyhow::Result;
 
-use distribution_types::{
-    CachedDist, IndexLocations, InstalledDist, Requirement, Resolution, SourceDist,
-};
+use distribution_types::{CachedDist, IndexLocations, InstalledDist, Resolution, SourceDist};
 use pep508_rs::PackageName;
+use pypi_types::Requirement;
 use uv_cache::Cache;
 use uv_configuration::{BuildKind, NoBinary, NoBuild, SetupPyStrategy};
-use uv_interpreter::{Interpreter, PythonEnvironment};
+use uv_git::GitResolver;
+use uv_toolchain::{Interpreter, PythonEnvironment};
 
 use crate::BuildIsolation;
 
@@ -56,6 +56,9 @@ pub trait BuildContext {
     /// Return a reference to the cache.
     fn cache(&self) -> &Cache;
 
+    /// Return a reference to the Git resolver.
+    fn git(&self) -> &GitResolver;
+
     /// All (potentially nested) source distribution builds use the same base python and can reuse
     /// it's metadata (e.g. wheel compatibility tags).
     fn interpreter(&self) -> &Interpreter;
