@@ -11,9 +11,9 @@ use uv_cache::CacheArgs;
 use uv_configuration::{
     ConfigSettingEntry, IndexStrategy, KeyringProviderType, PackageNameSpecifier, TargetTriple,
 };
-use uv_interpreter::PythonVersion;
 use uv_normalize::{ExtraName, PackageName};
 use uv_resolver::{AnnotationStyle, ExcludeNewer, PreReleaseMode, ResolutionMode};
+use uv_toolchain::PythonVersion;
 
 use crate::commands::{extra_name_with_clap_error, ListFormat, VersionFormat};
 use crate::compat;
@@ -130,6 +130,8 @@ pub(crate) enum Commands {
     Pip(PipNamespace),
     /// Run and manage executable Python packages.
     Tool(ToolNamespace),
+    /// Manage Python installations.
+    Toolchain(ToolchainNamespace),
     /// Create a virtual environment.
     #[command(alias = "virtualenv", alias = "v")]
     Venv(VenvArgs),
@@ -253,15 +255,30 @@ fn parse_index_url(input: &str) -> Result<Maybe<IndexUrl>, String> {
     }
 }
 
+/// Parse a string into a [`PathBuf`]. The string can represent a file, either as a path or a
+/// `file://` URL.
+fn parse_file_path(input: &str) -> Result<PathBuf, String> {
+    if input.starts_with("file://") {
+        let url = match url::Url::from_str(input) {
+            Ok(url) => url,
+            Err(err) => return Err(err.to_string()),
+        };
+        url.to_file_path()
+            .map_err(|()| "invalid file URL".to_string())
+    } else {
+        match PathBuf::from_str(input) {
+            Ok(path) => Ok(path),
+            Err(err) => Err(err.to_string()),
+        }
+    }
+}
+
 /// Parse a string into a [`PathBuf`], mapping the empty string to `None`.
-fn parse_file_path(input: &str) -> Result<Maybe<PathBuf>, String> {
+fn parse_maybe_file_path(input: &str) -> Result<Maybe<PathBuf>, String> {
     if input.is_empty() {
         Ok(Maybe::None)
     } else {
-        match PathBuf::from_str(input) {
-            Ok(path) => Ok(Maybe::Some(path)),
-            Err(err) => Err(err.to_string()),
-        }
+        parse_file_path(input).map(Maybe::Some)
     }
 }
 
@@ -270,8 +287,11 @@ fn parse_file_path(input: &str) -> Result<Maybe<PathBuf>, String> {
 pub(crate) struct PipCompileArgs {
     /// Include all packages listed in the given `requirements.in` files.
     ///
-    /// When the path is `-`, then requirements are read from stdin.
-    #[arg(required(true))]
+    /// If a `pyproject.toml`, `setup.py`, or `setup.cfg` file is provided, `uv` will
+    /// extract the requirements for the relevant project.
+    ///
+    /// If `-` is provided, then requirements will be read from stdin.
+    #[arg(required(true), value_parser = parse_file_path)]
     pub(crate) src_file: Vec<PathBuf>,
 
     /// Constrain versions using the given requirements files.
@@ -281,7 +301,7 @@ pub(crate) struct PipCompileArgs {
     /// trigger the installation of that package.
     ///
     /// This is equivalent to pip's `--constraint` option.
-    #[arg(long, short, env = "UV_CONSTRAINT", value_delimiter = ' ', value_parser = parse_file_path)]
+    #[arg(long, short, env = "UV_CONSTRAINT", value_delimiter = ' ', value_parser = parse_maybe_file_path)]
     pub(crate) constraint: Vec<Maybe<PathBuf>>,
 
     /// Override versions using the given requirements files.
@@ -293,14 +313,16 @@ pub(crate) struct PipCompileArgs {
     /// While constraints are _additive_, in that they're combined with the requirements of the
     /// constituent packages, overrides are _absolute_, in that they completely replace the
     /// requirements of the constituent packages.
-    #[arg(long)]
+    #[arg(long, value_parser = parse_file_path)]
     pub(crate) r#override: Vec<PathBuf>,
 
-    /// Include optional dependencies in the given extra group name; may be provided more than once.
+    /// Include optional dependencies from the extra group name; may be provided more than once.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "all_extras", value_parser = extra_name_with_clap_error)]
     pub(crate) extra: Option<Vec<ExtraName>>,
 
     /// Include all optional dependencies.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "extra")]
     pub(crate) all_extras: bool,
 
@@ -593,7 +615,12 @@ pub(crate) struct PipCompileArgs {
 #[allow(clippy::struct_excessive_bools)]
 pub(crate) struct PipSyncArgs {
     /// Include all packages listed in the given `requirements.txt` files.
-    #[arg(required(true))]
+    ///
+    /// If a `pyproject.toml`, `setup.py`, or `setup.cfg` file is provided, `uv` will
+    /// extract the requirements for the relevant project.
+    ///
+    /// If `-` is provided, then requirements will be read from stdin.
+    #[arg(required(true), value_parser = parse_file_path)]
     pub(crate) src_file: Vec<PathBuf>,
 
     /// Constrain versions using the given requirements files.
@@ -603,7 +630,7 @@ pub(crate) struct PipSyncArgs {
     /// trigger the installation of that package.
     ///
     /// This is equivalent to pip's `--constraint` option.
-    #[arg(long, short, env = "UV_CONSTRAINT", value_delimiter = ' ', value_parser = parse_file_path)]
+    #[arg(long, short, env = "UV_CONSTRAINT", value_delimiter = ' ', value_parser = parse_maybe_file_path)]
     pub(crate) constraint: Vec<Maybe<PathBuf>>,
 
     /// Reinstall all packages, regardless of whether they're already installed.
@@ -731,10 +758,21 @@ pub(crate) struct PipSyncArgs {
     pub(crate) no_break_system_packages: bool,
 
     /// Install packages into the specified directory, rather than into the virtual environment
-    /// or system Python interpreter.
-    #[arg(long)]
+    /// or system Python interpreter. The packages will be installed at the top-level of the
+    /// directory
+    #[arg(long, conflicts_with = "prefix")]
     pub(crate) target: Option<PathBuf>,
 
+    /// Install packages into `lib`, `bin`, and other top-level folders under the specified
+    /// directory, as if a virtual environment were created at the specified location.
+    ///
+    /// In general, prefer the use of `--python` to install into an alternate environment, as
+    /// scripts and other artifacts installed via `--prefix` will reference the installing
+    /// interpreter, rather than any interpreter added to the `--prefix` directory, rendering them
+    /// non-portable.
+    #[arg(long, conflicts_with = "target")]
+    pub(crate) prefix: Option<PathBuf>,
+
     /// Use legacy `setuptools` behavior when building source distributions without a
     /// `pyproject.toml`.
     #[arg(long, overrides_with("no_legacy_setup_py"))]
@@ -880,8 +918,13 @@ pub(crate) struct PipInstallArgs {
     #[arg(group = "sources")]
     pub(crate) package: Vec<String>,
 
-    /// Install all packages listed in the given requirements files.
-    #[arg(long, short, group = "sources")]
+    /// Install all packages listed in the given `requirements.txt` files.
+    ///
+    /// If a `pyproject.toml`, `setup.py`, or `setup.cfg` file is provided, `uv` will
+    /// extract the requirements for the relevant project.
+    ///
+    /// If `-` is provided, then requirements will be read from stdin.
+    #[arg(long, short, group = "sources", value_parser = parse_file_path)]
     pub(crate) requirement: Vec<PathBuf>,
 
     /// Install the editable package based on the provided local file path.
@@ -895,7 +938,7 @@ pub(crate) struct PipInstallArgs {
     /// trigger the installation of that package.
     ///
     /// This is equivalent to pip's `--constraint` option.
-    #[arg(long, short, env = "UV_CONSTRAINT", value_delimiter = ' ', value_parser = parse_file_path)]
+    #[arg(long, short, env = "UV_CONSTRAINT", value_delimiter = ' ', value_parser = parse_maybe_file_path)]
     pub(crate) constraint: Vec<Maybe<PathBuf>>,
 
     /// Override versions using the given requirements files.
@@ -907,14 +950,16 @@ pub(crate) struct PipInstallArgs {
     /// While constraints are _additive_, in that they're combined with the requirements of the
     /// constituent packages, overrides are _absolute_, in that they completely replace the
     /// requirements of the constituent packages.
-    #[arg(long)]
+    #[arg(long, value_parser = parse_file_path)]
     pub(crate) r#override: Vec<PathBuf>,
 
-    /// Include optional dependencies in the given extra group name; may be provided more than once.
+    /// Include optional dependencies from the extra group name; may be provided more than once.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "all_extras", value_parser = extra_name_with_clap_error)]
     pub(crate) extra: Option<Vec<ExtraName>>,
 
     /// Include all optional dependencies.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "extra", overrides_with = "no_all_extras")]
     pub(crate) all_extras: bool,
 
@@ -1087,10 +1132,21 @@ pub(crate) struct PipInstallArgs {
     pub(crate) no_break_system_packages: bool,
 
     /// Install packages into the specified directory, rather than into the virtual environment
-    /// or system Python interpreter.
-    #[arg(long)]
+    /// or system Python interpreter. The packages will be installed at the top-level of the
+    /// directory
+    #[arg(long, conflicts_with = "prefix")]
     pub(crate) target: Option<PathBuf>,
 
+    /// Install packages into `lib`, `bin`, and other top-level folders under the specified
+    /// directory, as if a virtual environment were created at the specified location.
+    ///
+    /// In general, prefer the use of `--python` to install into an alternate environment, as
+    /// scripts and other artifacts installed via `--prefix` will reference the installing
+    /// interpreter, rather than any interpreter added to the `--prefix` directory, rendering them
+    /// non-portable.
+    #[arg(long, conflicts_with = "target")]
+    pub(crate) prefix: Option<PathBuf>,
+
     /// Use legacy `setuptools` behavior when building source distributions without a
     /// `pyproject.toml`.
     #[arg(long, overrides_with("no_legacy_setup_py"))]
@@ -1237,7 +1293,7 @@ pub(crate) struct PipUninstallArgs {
     pub(crate) package: Vec<String>,
 
     /// Uninstall all packages listed in the given requirements files.
-    #[arg(long, short, group = "sources")]
+    #[arg(long, short, group = "sources", value_parser = parse_file_path)]
     pub(crate) requirement: Vec<PathBuf>,
 
     /// The Python interpreter from which packages should be uninstalled.
@@ -1300,10 +1356,13 @@ pub(crate) struct PipUninstallArgs {
     #[arg(long, overrides_with("break_system_packages"))]
     pub(crate) no_break_system_packages: bool,
 
-    /// Uninstall packages from the specified directory, rather than from the virtual environment
-    /// or system Python interpreter.
-    #[arg(long)]
+    /// Uninstall packages from the specified `--target` directory.
+    #[arg(long, conflicts_with = "prefix")]
     pub(crate) target: Option<PathBuf>,
+
+    /// Uninstall packages from the specified `--prefix` directory.
+    #[arg(long, conflicts_with = "target")]
+    pub(crate) prefix: Option<PathBuf>,
 }
 
 #[derive(Args)]
@@ -1654,17 +1713,27 @@ pub(crate) struct VenvArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub(crate) struct RunArgs {
-    /// Include optional dependencies in the given extra group name; may be provided more than once.
+    /// Include optional dependencies from the extra group name; may be provided more than once.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "all_extras", value_parser = extra_name_with_clap_error)]
     pub(crate) extra: Option<Vec<ExtraName>>,
 
     /// Include all optional dependencies.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "extra")]
     pub(crate) all_extras: bool,
 
     #[arg(long, overrides_with("all_extras"), hide = true)]
     pub(crate) no_all_extras: bool,
 
+    /// Include development dependencies.
+    #[arg(long, overrides_with("no_dev"), hide = true)]
+    pub(crate) dev: bool,
+
+    /// Omit development dependencies.
+    #[arg(long, overrides_with("dev"))]
+    pub(crate) no_dev: bool,
+
     /// The command to run.
     pub(crate) target: Option<String>,
 
@@ -1735,17 +1804,27 @@ pub(crate) struct RunArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub(crate) struct SyncArgs {
-    /// Include optional dependencies in the given extra group name; may be provided more than once.
+    /// Include optional dependencies from the extra group name; may be provided more than once.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "all_extras", value_parser = extra_name_with_clap_error)]
     pub(crate) extra: Option<Vec<ExtraName>>,
 
     /// Include all optional dependencies.
+    /// Only applies to `pyproject.toml`, `setup.py`, and `setup.cfg` sources.
     #[arg(long, conflicts_with = "extra")]
     pub(crate) all_extras: bool,
 
     #[arg(long, overrides_with("all_extras"), hide = true)]
     pub(crate) no_all_extras: bool,
 
+    /// Include development dependencies.
+    #[arg(long, overrides_with("no_dev"), hide = true)]
+    pub(crate) dev: bool,
+
+    /// Omit development dependencies.
+    #[arg(long, overrides_with("dev"))]
+    pub(crate) no_dev: bool,
+
     /// Refresh all cached data.
     #[arg(long, conflicts_with("offline"), overrides_with("no_refresh"))]
     pub(crate) refresh: bool,
@@ -1899,6 +1978,42 @@ pub(crate) struct ToolRunArgs {
     pub(crate) python: Option<String>,
 }
 
+#[derive(Args)]
+pub(crate) struct ToolchainNamespace {
+    #[command(subcommand)]
+    pub(crate) command: ToolchainCommand,
+}
+
+#[derive(Subcommand)]
+pub(crate) enum ToolchainCommand {
+    /// List the available toolchains.
+    List(ToolchainListArgs),
+
+    /// Download and install a specific toolchain.
+    Install(ToolchainInstallArgs),
+}
+
+#[derive(Args)]
+#[allow(clippy::struct_excessive_bools)]
+pub(crate) struct ToolchainListArgs {
+    /// List all available toolchains, including those that do not match the current platform.
+    #[arg(long, conflicts_with = "only_installed")]
+    pub(crate) all: bool,
+
+    /// Only list installed toolchains.
+    #[arg(long, conflicts_with = "all")]
+    pub(crate) only_installed: bool,
+}
+
+#[derive(Args)]
+#[allow(clippy::struct_excessive_bools)]
+pub(crate) struct ToolchainInstallArgs {
+    /// The toolchain to fetch.
+    ///
+    /// If not provided, the latest available version will be installed.
+    pub(crate) target: Option<String>,
+}
+
 #[derive(Args)]
 pub(crate) struct IndexArgs {
     /// The URL of the Python package index (by default: <https://pypi.org/simple>).
