@@ -9,13 +9,13 @@ use tokio::process::Command;
 use tracing::debug;
 
 use uv_cache::Cache;
-use uv_client::Connectivity;
+use uv_client::{BaseClientBuilder, Connectivity};
 use uv_configuration::{ExtrasSpecification, PreviewMode, Upgrade};
 use uv_distribution::{ProjectWorkspace, Workspace};
-use uv_interpreter::{PythonEnvironment, SystemPython};
 use uv_normalize::PackageName;
 use uv_requirements::RequirementsSource;
 use uv_resolver::ExcludeNewer;
+use uv_toolchain::{PythonEnvironment, SystemPython, Toolchain};
 use uv_warnings::warn_user;
 
 use crate::commands::{project, ExitStatus};
@@ -26,6 +26,7 @@ use crate::printer::Printer;
 pub(crate) async fn run(
     index_locations: IndexLocations,
     extras: ExtrasSpecification,
+    dev: bool,
     target: Option<String>,
     mut args: Vec<OsString>,
     requirements: Vec<RequirementsSource>,
@@ -39,6 +40,8 @@ pub(crate) async fn run(
     cache: &Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
+    let client_builder = BaseClientBuilder::new().connectivity(connectivity);
+
     if preview.is_disabled() {
         warn_user!("`uv run` is experimental and may change without warning.");
     }
@@ -60,12 +63,25 @@ pub(crate) async fn run(
         } else {
             ProjectWorkspace::discover(&std::env::current_dir()?, None).await?
         };
-        let venv = project::init_environment(&project, preview, cache, printer)?;
+        let venv = project::init_environment(
+            project.workspace(),
+            python.as_deref(),
+            preview,
+            cache,
+            printer,
+        )?;
 
         // Lock and sync the environment.
+        let root_project_name = project
+            .current_project()
+            .pyproject_toml()
+            .project
+            .as_ref()
+            .map(|project| project.name.clone());
         let lock = project::lock::do_lock(
-            &project,
-            &venv,
+            root_project_name,
+            project.workspace(),
+            venv.interpreter(),
             &index_locations,
             upgrade,
             exclude_newer,
@@ -80,6 +96,7 @@ pub(crate) async fn run(
             &lock,
             &index_locations,
             extras,
+            dev,
             preview,
             cache,
             printer,
@@ -99,11 +116,17 @@ pub(crate) async fn run(
         // Discover an interpreter.
         let interpreter = if let Some(project_env) = &project_env {
             project_env.interpreter().clone()
-        } else if let Some(python) = python.as_ref() {
-            PythonEnvironment::from_requested_python(python, SystemPython::Allowed, preview, cache)?
-                .into_interpreter()
         } else {
-            PythonEnvironment::from_default_python(preview, cache)?.into_interpreter()
+            // Note we force preview on during `uv run` for now since the entire interface is in preview
+            Toolchain::find_or_fetch(
+                python.as_deref(),
+                SystemPython::Allowed,
+                PreviewMode::Enabled,
+                client_builder,
+                cache,
+            )
+            .await?
+            .into_interpreter()
         };
 
         // TODO(charlie): If the environment satisfies the requirements, skip creation.
