@@ -17,10 +17,8 @@ use uv_configuration::PreviewMode;
 
 use uv_cache::Cache;
 use uv_fs::Simplified;
-use uv_interpreter::managed::InstalledToolchains;
-use uv_interpreter::{
-    find_interpreter, InterpreterRequest, PythonVersion, SourceSelector, VersionRequest,
-};
+use uv_toolchain::managed::InstalledToolchains;
+use uv_toolchain::{PythonVersion, Toolchain};
 
 // Exclude any packages uploaded after this date.
 pub static EXCLUDE_NEWER: &str = "2024-03-25T00:00:00Z";
@@ -28,7 +26,7 @@ pub static EXCLUDE_NEWER: &str = "2024-03-25T00:00:00Z";
 /// Using a find links url allows using `--index-url` instead of `--extra-index-url` in tests
 /// to prevent dependency confusion attacks against our test suite.
 pub const BUILD_VENDOR_LINKS_URL: &str =
-    "https://raw.githubusercontent.com/astral-sh/packse/0.3.15/vendor/links.html";
+    "https://raw.githubusercontent.com/astral-sh/packse/0.3.18/vendor/links.html";
 
 #[doc(hidden)] // Macro and test context only, don't use directly.
 pub const INSTA_FILTERS: &[(&str, &str)] = &[
@@ -74,7 +72,7 @@ impl TestContext {
             .expect("CARGO_MANIFEST_DIR should be doubly nested in workspace")
             .to_path_buf();
 
-        let site_packages = site_packages_path(&venv, format!("python{python_version}"));
+        let site_packages = site_packages_path(&venv, &format!("python{python_version}"));
 
         let python_version =
             PythonVersion::from_str(python_version).expect("Tests must use valid Python versions");
@@ -224,6 +222,94 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv sync` command with options shared across scenarios.
+    pub fn sync(&self) -> std::process::Command {
+        let mut command = std::process::Command::new(get_bin());
+        command
+            .arg("sync")
+            .arg("--cache-dir")
+            .arg(self.cache_dir.path())
+            .env("VIRTUAL_ENV", self.venv.as_os_str())
+            .env("UV_NO_WRAP", "1")
+            .current_dir(&self.temp_dir);
+
+        if cfg!(all(windows, debug_assertions)) {
+            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
+            // default windows stack of 1MB
+            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
+        }
+
+        command
+    }
+
+    /// Create a `uv lock` command with options shared across scenarios.
+    pub fn lock(&self) -> std::process::Command {
+        let mut command = self.lock_without_exclude_newer();
+        command.arg("--exclude-newer").arg(EXCLUDE_NEWER);
+        command
+    }
+
+    /// Create a `uv lock` command with no `--exclude-newer` option.
+    ///
+    /// One should avoid using this in tests to the extent possible because
+    /// it can result in tests failing when the index state changes. Therefore,
+    /// if you use this, there should be some other kind of mitigation in place.
+    /// For example, pinning package versions.
+    pub fn lock_without_exclude_newer(&self) -> std::process::Command {
+        let mut command = std::process::Command::new(get_bin());
+        command
+            .arg("lock")
+            .arg("--cache-dir")
+            .arg(self.cache_dir.path())
+            .env("VIRTUAL_ENV", self.venv.as_os_str())
+            .env("UV_NO_WRAP", "1")
+            .current_dir(&self.temp_dir);
+
+        if cfg!(all(windows, debug_assertions)) {
+            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
+            // default windows stack of 1MB
+            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
+        }
+
+        command
+    }
+
+    /// Create a `uv run` command with options shared across scenarios.
+    pub fn run(&self) -> std::process::Command {
+        let mut command = self.run_without_exclude_newer();
+        command.arg("--exclude-newer").arg(EXCLUDE_NEWER);
+        command
+    }
+
+    /// Create a `uv run` command with no `--exclude-newer` option.
+    ///
+    /// One should avoid using this in tests to the extent possible because
+    /// it can result in tests failing when the index state changes. Therefore,
+    /// if you use this, there should be some other kind of mitigation in place.
+    /// For example, pinning package versions.
+    pub fn run_without_exclude_newer(&self) -> std::process::Command {
+        let mut command = std::process::Command::new(get_bin());
+        command
+            .arg("run")
+            .arg("--cache-dir")
+            .arg(self.cache_dir.path())
+            .env("VIRTUAL_ENV", self.venv.as_os_str())
+            .env("UV_NO_WRAP", "1")
+            .current_dir(&self.temp_dir);
+
+        if cfg!(all(windows, debug_assertions)) {
+            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
+            // default windows stack of 1MB
+            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
+        }
+
+        command
+    }
+
+    pub fn interpreter(&self) -> PathBuf {
+        venv_to_interpreter(&self.venv)
+    }
+
     /// Run the given python code and check whether it succeeds.
     pub fn assert_command(&self, command: &str) -> Assert {
         std::process::Command::new(venv_to_interpreter(&self.venv))
@@ -257,7 +343,7 @@ impl TestContext {
     }
 
     /// Generate various escaped regex patterns for the given path.
-    fn path_patterns(path: impl AsRef<Path>) -> Vec<String> {
+    pub(crate) fn path_patterns(path: impl AsRef<Path>) -> Vec<String> {
         let mut patterns = Vec::new();
 
         // We can only canonicalize paths that exist already
@@ -308,12 +394,12 @@ impl TestContext {
     pub fn site_packages(&self) -> PathBuf {
         site_packages_path(
             &self.venv,
-            format!("{}{}", self.python_kind(), self.python_version),
+            &format!("{}{}", self.python_kind(), self.python_version),
         )
     }
 }
 
-fn site_packages_path(venv: &Path, python: String) -> PathBuf {
+pub fn site_packages_path(venv: &Path, python: &str) -> PathBuf {
     if cfg!(unix) {
         venv.join("lib").join(python).join("site-packages")
     } else if cfg!(windows) {
@@ -343,14 +429,9 @@ pub fn venv_to_interpreter(venv: &Path) -> PathBuf {
     }
 }
 
-/// Create a virtual environment named `.venv` in a temporary directory with the given
-/// Python version. Expected format for `python` is "<version>".
-pub fn create_venv<Parent: assert_fs::prelude::PathChild + AsRef<std::path::Path>>(
-    temp_dir: &Parent,
-    cache_dir: &assert_fs::TempDir,
-    python: &str,
-) -> PathBuf {
-    let python = InstalledToolchains::from_settings()
+/// Get the path to the python interpreter for a specific toolchain version.
+pub fn get_toolchain(python: &str) -> PathBuf {
+    InstalledToolchains::from_settings()
         .map(|installed_toolchains| {
             installed_toolchains
                 .find_version(
@@ -360,12 +441,22 @@ pub fn create_venv<Parent: assert_fs::prelude::PathChild + AsRef<std::path::Path
                 .expect("Tests are run on a supported platform")
                 .next()
                 .as_ref()
-                .map(uv_interpreter::managed::Toolchain::executable)
+                .map(uv_toolchain::managed::InstalledToolchain::executable)
         })
         // We'll search for the request Python on the PATH if not found in the toolchain versions
         // We hack this into a `PathBuf` to satisfy the compiler but it's just a string
         .unwrap_or_default()
-        .unwrap_or(PathBuf::from(python));
+        .unwrap_or(PathBuf::from(python))
+}
+
+/// Create a virtual environment named `.venv` in a temporary directory with the given
+/// Python version. Expected format for `python` is "<version>".
+pub fn create_venv<Parent: assert_fs::prelude::PathChild + AsRef<std::path::Path>>(
+    temp_dir: &Parent,
+    cache_dir: &assert_fs::TempDir,
+    python: &str,
+) -> PathBuf {
+    let python = get_toolchain(python);
 
     let venv = temp_dir.child(".venv");
     Command::new(get_bin())
@@ -420,22 +511,15 @@ pub fn python_path_with_versions(
                 .unwrap_or_default();
             if inner.is_empty() {
                 // Fallback to a system lookup if we failed to find one in the toolchain directory
-                let request = InterpreterRequest::Version(
-                    VersionRequest::from_str(python_version)
-                        .expect("The test version request must be valid"),
-                );
-                let sources = SourceSelector::All(PreviewMode::Enabled);
-                if let Ok(found) = find_interpreter(
-                    &request,
+                if let Ok(toolchain) = Toolchain::find(
+                    Some(python_version),
                     // Without required, we could pick the current venv here and the test fails
                     // because the venv subcommand requires a system interpreter.
-                    uv_interpreter::SystemPython::Required,
-                    &sources,
+                    uv_toolchain::SystemPython::Required,
+                    PreviewMode::Enabled,
                     &cache,
-                )
-                .unwrap()
-                {
-                    vec![found
+                ) {
+                    vec![toolchain
                         .into_interpreter()
                         .sys_executable()
                         .parent()
@@ -458,6 +542,12 @@ pub fn python_path_with_versions(
     Ok(env::join_paths(selected_pythons)?)
 }
 
+#[derive(Debug, Copy, Clone)]
+pub enum WindowsFilters {
+    Platform,
+    Universal,
+}
+
 /// Execute the command and format its output status, stdout and stderr into a snapshot string.
 ///
 /// This function is derived from `insta_cmd`s `spawn_with_info`.
@@ -465,7 +555,7 @@ pub fn run_and_format<T: AsRef<str>>(
     mut command: impl BorrowMut<std::process::Command>,
     filters: impl AsRef<[(T, T)]>,
     function_name: &str,
-    windows_filters: bool,
+    windows_filters: Option<WindowsFilters>,
 ) -> (String, Output) {
     let program = command
         .borrow_mut()
@@ -510,29 +600,40 @@ pub fn run_and_format<T: AsRef<str>>(
     // pass whether it's on Windows or Unix. In particular, there are some very
     // common Windows-only dependencies that, when removed from a resolution,
     // cause the set of dependencies to be the same across platforms.
-    if cfg!(windows) && windows_filters {
-        // The optional leading +/- is for install logs, the optional next line is for lock files
-        let windows_only_deps = [
-            ("( [+-] )?colorama==\\d+(\\.[\\d+])+\n(    # via .*\n)?"),
-            ("( [+-] )?colorama==\\d+(\\.[\\d+])+\\s+(# via .*\n)?"),
-            ("( [+-] )?tzdata==\\d+(\\.[\\d+])+\n(    # via .*\n)?"),
-            ("( [+-] )?tzdata==\\d+(\\.[\\d+])+\\s+(# via .*\n)?"),
-        ];
-        let mut removed_packages = 0;
-        for windows_only_dep in windows_only_deps {
-            // TODO(konstin): Cache regex compilation
-            let re = Regex::new(windows_only_dep).unwrap();
-            if re.is_match(&snapshot) {
-                snapshot = re.replace(&snapshot, "").to_string();
-                removed_packages += 1;
+    if cfg!(windows) {
+        if let Some(windows_filters) = windows_filters {
+            // The optional leading +/- is for install logs, the optional next line is for lock files
+            let windows_only_deps = [
+                ("( [+-] )?colorama==\\d+(\\.[\\d+])+\n(    # via .*\n)?"),
+                ("( [+-] )?colorama==\\d+(\\.[\\d+])+\\s+(# via .*\n)?"),
+                ("( [+-] )?tzdata==\\d+(\\.[\\d+])+\n(    # via .*\n)?"),
+                ("( [+-] )?tzdata==\\d+(\\.[\\d+])+\\s+(# via .*\n)?"),
+            ];
+            let mut removed_packages = 0;
+            for windows_only_dep in windows_only_deps {
+                // TODO(konstin): Cache regex compilation
+                let re = Regex::new(windows_only_dep).unwrap();
+                if re.is_match(&snapshot) {
+                    snapshot = re.replace(&snapshot, "").to_string();
+                    removed_packages += 1;
+                }
             }
-        }
-        if removed_packages > 0 {
-            for i in 1..20 {
-                snapshot = snapshot.replace(
-                    &format!("{} packages", i + removed_packages),
-                    &format!("{} package{}", i, if i > 1 { "s" } else { "" }),
-                );
+            if removed_packages > 0 {
+                for i in 1..20 {
+                    for verb in match windows_filters {
+                        WindowsFilters::Platform => {
+                            ["Resolved", "Downloaded", "Installed", "Uninstalled"].iter()
+                        }
+                        WindowsFilters::Universal => {
+                            ["Downloaded", "Installed", "Uninstalled"].iter()
+                        }
+                    } {
+                        snapshot = snapshot.replace(
+                            &format!("{verb} {} packages", i + removed_packages),
+                            &format!("{verb} {} package{}", i, if i > 1 { "s" } else { "" }),
+                        );
+                    }
+                }
             }
         }
     }
@@ -555,6 +656,21 @@ pub fn copy_dir_all(src: impl AsRef<Path>, dst: impl AsRef<Path>) -> std::io::Re
     Ok(())
 }
 
+/// Recursively copy a directory and its contents, skipping gitignored files.
+pub fn copy_dir_ignore(src: impl AsRef<Path>, dst: impl AsRef<Path>) -> anyhow::Result<()> {
+    for entry in ignore::Walk::new(&src) {
+        let entry = entry?;
+        let relative = entry.path().strip_prefix(&src)?;
+        let ty = entry.file_type().unwrap();
+        if ty.is_dir() {
+            fs_err::create_dir(dst.as_ref().join(relative))?;
+        } else {
+            fs_err::copy(entry.path(), dst.as_ref().join(relative))?;
+        }
+    }
+    Ok(())
+}
+
 /// Utility macro to return the name of the current function.
 ///
 /// https://stackoverflow.com/a/40234666/3549270
@@ -585,13 +701,19 @@ macro_rules! uv_snapshot {
     }};
     ($filters:expr, $spawnable:expr, @$snapshot:literal) => {{
         // Take a reference for backwards compatibility with the vec-expecting insta filters.
-        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, function_name!(), true);
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, function_name!(), Some($crate::common::WindowsFilters::Platform));
         ::insta::assert_snapshot!(snapshot, @$snapshot);
         output
     }};
     ($filters:expr, windows_filters=false, $spawnable:expr, @$snapshot:literal) => {{
         // Take a reference for backwards compatibility with the vec-expecting insta filters.
-        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, function_name!(), false);
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, function_name!(), None);
+        ::insta::assert_snapshot!(snapshot, @$snapshot);
+        output
+    }};
+    ($filters:expr, universal_windows_filters=true, $spawnable:expr, @$snapshot:literal) => {{
+        // Take a reference for backwards compatibility with the vec-expecting insta filters.
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, function_name!(), Some($crate::common::WindowsFilters::Universal));
         ::insta::assert_snapshot!(snapshot, @$snapshot);
         output
     }};
